@@ -1,7 +1,23 @@
-# coding: utf-8
+# =========================================================================
+# Copyright 2012-present Yunify, Inc.
+# -------------------------------------------------------------------------
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this work except in compliance with the License.
+# You may obtain a copy of the License in the LICENSE file, or at:
+#
+#  http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+# =========================================================================
 
 import json
 
+from qingcloud.iaas.errors import InvalidSecurityGroupRule
+
 class SecurityGroupRuleFactory(object):
 
     PROTOCOL_TCP = 'tcp'
@@ -9,16 +25,24 @@ class SecurityGroupRuleFactory(object):
     PROTOCOL_ICMP = 'icmp'
     PROTOCOL_GRE = 'gre'
 
+    INBOUND = 0
+    OUTBOUND = 1
+
     @classmethod
-    def create(cls, protocol, priority, direction=0, action='accept',
+    def create(cls, protocol, priority, direction=INBOUND, action='accept',
             security_group_rule_id='', security_group_rule_name='',
             **kw):
-        clazz = RULE_MAPPER.get(protocol)
-        if not clazz:
-            return None
-        if not isinstance(priority, int) or priority < 0 or priority > 100:
-            return None
+        """ Create security group rule.
 
+            @param protocol: support protocol.
+            @param priority: should be between 0 and 100.
+        """
+        if protocol not in RULE_MAPPER:
+            raise InvalidSecurityGroupRule("invalid protocol[%s]" % protocol)
+        if not isinstance(priority, int) or priority < 0 or priority > 100:
+            raise InvalidSecurityGroupRule("invalid priority[%s]" % priority)
+
+        clazz = RULE_MAPPER[protocol]
         inst = clazz(**kw)
         inst.priority = priority
         inst.direction = direction
@@ -29,6 +53,8 @@ class SecurityGroupRuleFactory(object):
 
     @classmethod
     def create_from_string(cls, string):
+        """ Create security group rule from json formatted string.
+        """
         if not isinstance(string, basestring):
             return string
         data = json.loads(string)
@@ -39,14 +65,13 @@ class SecurityGroupRuleFactory(object):
 
 
 class _SecurityGroupRule(object):
-    """
-    _SecurityGroupRule is used to define a rule in security group.
+    """ _SecurityGroupRule is used to define a rule in security group.
     """
 
     security_group_rule_id = None
     security_group_rule_name = None
     priority = None
-    direction = None # 0: inbound, 1: outbound
+    direction = None
     action = None
     protocol = None
 
